import { FillType, } from "../interfaces";
import { isDefined } from "./TypeGuards";
// Lowest ConfigStore version where the V3 model is in effect. The version update to the following value should
// take place atomically with the SpokePool upgrade to V3 so that the dataworker knows what kind of MerkleLeaves
// to propose in root bundles (i.e. RelayerRefundLeaf and SlowFillLeaf have different shapes). We assume that
// V3 will be deployed in between bundles (after a bundle execution and before a proposal). The dataworker/relayer
// code can use the following isV3() function to separate logic for calling V3 vs. legacy methods.
export var V3_MIN_CONFIG_STORE_VERSION = 3;
export function isV3(version) {
    return version >= V3_MIN_CONFIG_STORE_VERSION;
}
export function isV2Deposit(deposit) {
    return isDefined(deposit.originToken);
}
export function isV3Deposit(deposit) {
    return isDefined(deposit.inputToken);
}
export function isV2SpeedUp(speedUp) {
    return isDefined(speedUp.newRelayerFeePct);
}
export function isV3SpeedUp(speedUp) {
    return isDefined(speedUp.updatedOutputAmount);
}
export function isV2Fill(fill) {
    return isDefined(fill.destinationToken);
}
export function isV3Fill(fill) {
    return isDefined(fill.inputToken);
}
export function isV2RelayData(relayData) {
    return isDefined(relayData.destinationToken);
}
export function isV3RelayData(relayData) {
    return isDefined(relayData.outputToken);
}
export function isSlowFill(fill) {
    return isV2Fill(fill) ? fill.updatableRelayData.isSlowRelay : fill.updatableRelayData.fillType === FillType.SlowFill;
}
export function isV2SlowFillLeaf(slowFillLeaf) {
    return isDefined(slowFillLeaf.payoutAdjustmentPct) && isV2RelayData(slowFillLeaf.relayData);
}
export function isV3SlowFillLeaf(slowFillLeaf) {
    return isDefined(slowFillLeaf.updatedOutputAmount) && isV3RelayData(slowFillLeaf.relayData);
}
export function getDepositInputToken(deposit) {
    return isV2Deposit(deposit) ? deposit.originToken : deposit.inputToken;
}
export function getDepositOutputToken(deposit) {
    return isV2Deposit(deposit) ? deposit.destinationToken : deposit.outputToken;
}
export function getFillOutputToken(fill) {
    return isV2Fill(fill) ? fill.destinationToken : fill.outputToken;
}
export function getDepositInputAmount(deposit) {
    return isV2Deposit(deposit) ? deposit.amount : deposit.inputAmount;
}
export function getDepositOutputAmount(deposit) {
    return isV2Deposit(deposit) ? deposit.amount : deposit.outputAmount;
}
export function getFillOutputAmount(fill) {
    return isV2Fill(fill) ? fill.amount : fill.outputAmount;
}
export function getFillAmount(fill) {
    return isV2Fill(fill) ? fill.fillAmount : fill.outputAmount;
}
export function getTotalFilledAmount(fill) {
    return isV2Fill(fill) ? fill.totalFilledAmount : fill.outputAmount;
}
export function getRelayDataOutputToken(relayData) {
    return isV2RelayData(relayData) ? relayData.destinationToken : relayData.outputToken;
}
export function getRelayDataOutputAmount(relayData) {
    return isV2RelayData(relayData) ? relayData.amount : relayData.outputAmount;
}
export function getSlowFillLeafChainId(leaf) {
    return isV2SlowFillLeaf(leaf) ? leaf.relayData.destinationChainId : leaf.chainId;
}
//# sourceMappingURL=V3Utils.js.map